#set master_claim_path ""
#set selected_jtag
global master_claim_path
global selected_jtag
global jtag_master_list
global jtag_master

set jtag_master 0

#Procedure for Widgets
proc widget_group {add name group itemsPerRow text} {
	global dash
	if {$add} {
		dashboard_add $dash $name group $group
	}
	dashboard_set_property $dash $name itemsPerRow $itemsPerRow
	dashboard_set_property $dash $name title $text
} 

proc widget_groupHeight {add name group itemsPerRow text preferredHeight} {
	global dash
	if {$add} {
		dashboard_add $dash $name group $group
	}
	dashboard_set_property $dash $name itemsPerRow $itemsPerRow
	dashboard_set_property $dash $name title $text
	dashboard_set_property $dash $name preferredHeight $preferredHeight
} 

proc widget_groupWidth {add name group itemsPerRow text preferredWidth} {
	global dash
	if {$add} {
		dashboard_add $dash $name group $group
	}
	dashboard_set_property $dash $name itemsPerRow $itemsPerRow
	dashboard_set_property $dash $name title $text
	dashboard_set_property $dash $name preferredWidth $preferredWidth
}

proc widget_comboBox {add name group options} {
	global dash
	if {$add} {
		dashboard_add $dash $name comboBox $group
	}
	dashboard_set_property $dash $name options $options
	
}

proc widget_comboBoxonChange {add name group options onChange} {
	global dash
	if {$add} {
		dashboard_add $dash $name comboBox $group
	}
	dashboard_set_property $dash $name options $options
	dashboard_set_property $dash $name onChange $onChange
}

proc widget_button {add name group text onClick} {
	global dash
	if {$add} {
		dashboard_add $dash $name button $group
	}
	dashboard_set_property $dash $name text $text
	#dashboard_set_property $dash $name enabled $enabled
	#dashboard_set_property $dash $name backgroundColor $backgroundColor
	#dashboard_set_property $dash $name foregroundColor $foregroundColor
	#dashboard_set_property $dash $name preferredWidth $preferredWidth
	dashboard_set_property $dash $name onClick $onClick
}

proc widget_textBox {add name group editable text preferredHeight preferredWidth} {
	global dash
	if {$add} {
		dashboard_add $dash $name text $group
	}
	dashboard_set_property $dash $name editable $editable
	dashboard_set_property $dash $name text $text
	dashboard_set_property $dash $name preferredHeight $preferredHeight
    dashboard_set_property $dash $name preferredWidth $preferredWidth
}

proc widget_label {add name group text} {
	global dash
	if {$add} {
		dashboard_add $dash $name label $group
	}
	dashboard_set_property $dash $name text $text
	#dashboard_set_property $dash $name foregroundColor $foregroundColor
	#dashboard_set_property $dash $name preferredWidth 50
}

proc widget_labelwidth {add name group text width} {
	global dash
	if {$add} {
		dashboard_add $dash $name label $group
	}
	dashboard_set_property $dash $name text $text
	#dashboard_set_property $dash $name foregroundColor $foregroundColor
	dashboard_set_property $dash $name preferredWidth $width
}

proc widget_checkBox {add name group text onClick} {
	global dash
	if {$add} {
		dashboard_add $dash $name checkBox $group
	}
	dashboard_set_property $dash $name text $text
	dashboard_set_property $dash $name onClick $onClick
}

proc widget_led {add name group text color} {
	global dash
	if {$add} {
		dashboard_add $dash $name led $group
	}
	dashboard_set_property $dash $name text $text
	dashboard_set_property $dash $name color $color
}

#jtag procedures

proc open_port {} {
    global master_claim_path
	global jtag_master
    variable jtag_master_list
    variable selected_jtag

    set port_id [lindex $jtag_master_list $selected_jtag]
    set jtag_master [lindex $jtag_master_list $selected_jtag]
    set master_claim_path [claim_service master $port_id mylib]
}

proc close_port {} {
    global master_claim_path
    close_service master $master_claim_path
}

proc list_jtag_proc {} {
    global dash
	global jtag_master_list
    global selected_jtag

    if {![info exists selected_jtag]} {set selected_jtag 0}

    set jtag_master_list [get_service_paths master]
    set n 0

    puts "Available JTAG Masters:"

	set myjtag_list ""
	set setjtag_list ""
    foreach {path} $jtag_master_list {
        puts "$n: $path"
		append setjtag_list " $n"
		if {([regexp "devices/1S" $path match] || [regexp "devices/432" $path match]) & ![regexp "CJTAG" $path match]} {
		append myjtag_list "$n: $path \n    -> [marker_get_info $path] \n \n"
		} else {
		append myjtag_list "$n: $path \n \n" 
		}
        incr n
    }

	widget_textBox 0 jtag_master list_jtag_master 0 $myjtag_list 160 1100
	dashboard_set_property $dash set_jtag_value options $setjtag_list
	
	open_port
	
}

proc set_jtag_proc {} {
	global dash
    global selected_jtag
    global jtag_master_list
	global set_jtag_master
    close_port
    
	set set_jtag_master 1
	
	set selected_jtag [dashboard_get_property $dash set_jtag_value selectedItem]
    puts "Currently selected master is $selected_jtag:"
	puts [lindex $jtag_master_list $selected_jtag]
	dashboard_set_property $dash selected_jtag_master_2 text "$selected_jtag"
	dashboard_set_property $dash selected_jtag_master_3 text [lindex $jtag_master_list $selected_jtag]
    widget_label 0 set_jtag_note select_ip_option "3. Select the JTAG Master number from the drop down menu and click 'Set JTAG Master'"
	dashboard_set_property $dash set_jtag_note foregroundColor blue
	
    open_port
}


#Reg Read and Write

#______________________________________________________________________________
# write : 32-bit data with 24-bit addresses
#______________________________________________________________________________
proc reg_write {base args} {
    global master_claim_path

    set arg_list [regexp -all -inline {\S+} $args]
    set num_args [llength $arg_list]

    if {$num_args == 1} {
        set offset 0
        set wdata [lindex $arg_list 0]
    } elseif {$num_args == 2} {
        set offset [lindex $arg_list 0]
        set wdata  [lindex $arg_list 1]
    } else {
        puts "reg_write: Wrong number of arguments"
        return -1
    }

    set word_address [ expr $base + $offset ]
    set byte_address [ expr $word_address * 4 ]
    master_write_32 $master_claim_path $byte_address $wdata
    return 0
}

#______________________________________________________________________________
# read : 32-bit data with 24-bit addresses
#______________________________________________________________________________
 proc reg_read {base {offset 0}} {
    global master_claim_path

    set word_address [ expr $base + $offset ]
    set byte_address [ expr $word_address * 4 ]
    set rdata [master_read_32 $master_claim_path $byte_address 1]
    return $rdata
}

proc reg_rmw_adp {addr offset mask val} {
    set value [reg_read $addr $offset]
    set value [expr $value & [expr 0xffffffff & ~$mask]] 
    set value [expr $value | $val] 
    reg_write $addr $offset $value
}

#Packet Generator

proc packet_generator {} {

	global dash
		widget_group 1 PKT_STATS_CNTRL PKT_STATS_COUNT 1 "Packet Generator - only available when using Example Design"
				
					widget_group 1 pkt_cont  PKT_STATS_CNTRL 2 ""
					
					widget_checkBox 1 pkt_cntrl_en pkt_cont "Quartus generated Example Design used" proc_pkt_cntrl_en
					dashboard_set_property $dash pkt_cntrl_en toolTip "When using Example Design, select JTAG Master with name 'jtag_master' (and not 'jtag_master_alt_e100s10') to use packet generator"
					widget_checkBox 1  mac_loopback pkt_cont "MAC Loopback Mode (RX AVST --> TX AVST)" proc_mac_loopback
					dashboard_set_property $dash mac_loopback toolTip "When using this feature, the received data on RX AvST client interface is looped back to TX AvST client interface"
					dashboard_set_property $dash mac_loopback enabled false	
					
					widget_group 1 Source_Dest_Address PKT_STATS_CNTRL 3 ""
						
						widget_button 1 set_source_add Source_Dest_Address "Set Source Address      " proc_set_source_add
						dashboard_set_property $dash set_source_add enabled false	
						widget_textBox 1 source_Add Source_Dest_Address 0 "0x976543210ADD" 10 150
						widget_label 1 range_source Source_Dest_Address "Range: 0x0 to 0xFFFFFFFFFFFF"
						
						widget_button 1 set_dest_add Source_Dest_Address "Set Destination Address" proc_set_dest_add
						dashboard_set_property $dash set_dest_add enabled false	
						widget_textBox 1 dest_Add Source_Dest_Address 0 "0x123456780ADD" 10 150
						widget_label 1 range_dest Source_Dest_Address "Range: 0x0 to 0xFFFFFFFFFFFF"
										
						
					widget_group 1 PKT_CNTRL PKT_STATS_CNTRL 3 ""
					
								
					widget_group 1  random_mode PKT_CNTRL 2 "Random Mode"
						widget_label 1  random_gap random_mode "Random Gap"
						widget_button 1 pkt_gen_random_gap random_mode "Start Packet Generator" proc_pkt_gen_random_gap
						dashboard_set_property $dash pkt_gen_random_gap enabled false	
						
						widget_label 1  no_gap random_mode "No Gap (Full Rate)"
						widget_button 1 pkt_gen_no_gap random_mode "Start Packet Generator" proc_pkt_gen_no_gap
						dashboard_set_property $dash pkt_gen_no_gap enabled false	
						
					widget_group 1  fixed_mode PKT_CNTRL 3 "Fixed Mode (In Decimal)"
						widget_label 1  pkt_byte_size fixed_mode "Packet Byte Size"
						widget_textBox 1  pkt_byte_size_val fixed_mode 0 "64" 10 75
						widget_label 1  pkt_byte_range fixed_mode "Range 64 - 9600"
						
						widget_label 1  pkt_num fixed_mode "Total no. of Packets"
						widget_textBox 1  pkt_num_val fixed_mode 0 "10" 10 125
						widget_label 1  pkt_num_blank fixed_mode "Range 1 - 4294967295"
						
						widget_button 1 pkt_gen_fixed fixed_mode "Start Packet Generator" proc_pkt_gen_fixed
							dashboard_set_property $dash pkt_gen_fixed toolTip "Send the number of packets defined in 'Total no. of Packets' and cannot be stopped till it completes"
						dashboard_set_property $dash pkt_gen_fixed enabled false	
		
						
					widget_group 1  incr_mode PKT_CNTRL 3 "Incremental Mode (In Decimal)"
						widget_label 1  pkt_byte_size_begin incr_mode "Packet Byte Size - Begin"
						widget_textBox 1  pkt_byte_size_begin_val incr_mode 0 "64" 10 75
						widget_label 1  pkt_byte_begin_range incr_mode "Range 64 - 9599"
						
						widget_label 1  pkt_byte_size_end incr_mode "Packet Byte Size - End"
						widget_textBox 1  pkt_byte_size_end_val incr_mode 0 "9600" 10 75
						widget_label 1  pkt_byte_end_range incr_mode "Range 65 - 9600"
						
						widget_label 1  pkt_incr_num incr_mode "Total no. of Packets"
						widget_textBox 1  pkt_incr_num_val incr_mode 0 "10" 10 125
						widget_label 1  pkt_incr_num_blank incr_mode "Range 1 - 4294967295"
						
						widget_button 1 pkt_gen_incr incr_mode "Start Packet Generator" proc_pkt_gen_incr
							dashboard_set_property $dash pkt_gen_incr toolTip "Send the number of packets defined in 'Total no. of Packets' and cannot be stopped till it completes"
						dashboard_set_property $dash pkt_gen_incr enabled false
}


proc proc_pkt_cntrl_en {} {

	global dash
	
	if { [dashboard_get_property $dash pkt_cntrl_en checked] } {
		dashboard_set_property $dash mac_loopback enabled true
	} else {
		dashboard_set_property $dash mac_loopback enabled false
	}

	if { [string match [dashboard_get_property $dash pkt_cntrl_en checked] "true"] & [string match [dashboard_get_property $dash mac_loopback checked] "false"] } {
		
		dashboard_set_property $dash pkt_gen_random_gap enabled true	
		dashboard_set_property $dash pkt_gen_no_gap enabled true	
		dashboard_set_property $dash pkt_gen_fixed enabled true	
		dashboard_set_property $dash pkt_gen_incr enabled true	
		
		dashboard_set_property $dash set_source_add enabled true	
		dashboard_set_property $dash set_dest_add enabled true
		
		dashboard_set_property $dash source_Add editable 1 
		dashboard_set_property $dash dest_Add editable 1 
		dashboard_set_property $dash pkt_byte_size_val editable 1 
		dashboard_set_property $dash pkt_num_val editable 1 
		dashboard_set_property $dash pkt_byte_size_begin_val editable 1 
		dashboard_set_property $dash pkt_byte_size_end_val editable 1 
		dashboard_set_property $dash pkt_incr_num_val editable 1 
			
	}
	if { [string match [dashboard_get_property $dash pkt_cntrl_en checked] "false"] & [string match [dashboard_get_property $dash mac_loopback checked] "false"] } {
		
	
		dashboard_set_property $dash pkt_gen_random_gap enabled false	
		dashboard_set_property $dash pkt_gen_no_gap enabled false	
		dashboard_set_property $dash pkt_gen_fixed enabled false	
		dashboard_set_property $dash pkt_gen_incr enabled false			
		
		dashboard_set_property $dash set_source_add enabled false
		dashboard_set_property $dash set_dest_add enabled false	
		
		dashboard_set_property $dash source_Add editable 0 
		dashboard_set_property $dash dest_Add editable 0 
		dashboard_set_property $dash pkt_byte_size_val editable 0 
		dashboard_set_property $dash pkt_num_val editable 0 
		dashboard_set_property $dash pkt_byte_size_begin_val editable 0 
		dashboard_set_property $dash pkt_byte_size_end_val editable 0 
		dashboard_set_property $dash pkt_incr_num_val editable 0 
			
		
	}	
	
}

proc proc_mac_loopback {} {
	
	global dash
	
	if {[string match [dashboard_get_property $dash mac_loopback checked] "true"]} {
		
		reg_write 0x1010 0xF		
		
		dashboard_set_property $dash pkt_gen_random_gap enabled false	
		dashboard_set_property $dash pkt_gen_no_gap enabled false	
		dashboard_set_property $dash pkt_gen_fixed enabled false	
		dashboard_set_property $dash pkt_gen_incr enabled false			
		dashboard_set_property $dash set_source_add enabled false
		dashboard_set_property $dash set_dest_add enabled false
		
		dashboard_set_property $dash source_Add editable 0 
		dashboard_set_property $dash dest_Add editable 0 
		dashboard_set_property $dash pkt_byte_size_val editable 0 
		dashboard_set_property $dash pkt_num_val editable 0 
		dashboard_set_property $dash pkt_byte_size_begin_val editable 0 
		dashboard_set_property $dash pkt_byte_size_end_val editable 0 
		dashboard_set_property $dash pkt_incr_num_val editable 0 
			
						
	} else {
		
		reg_write 0x1010 0x0
		
		dashboard_set_property $dash pkt_gen_random_gap enabled true	
		dashboard_set_property $dash pkt_gen_no_gap enabled true	
		dashboard_set_property $dash pkt_gen_fixed enabled true	
		dashboard_set_property $dash pkt_gen_incr enabled true	
		dashboard_set_property $dash set_source_add enabled true	
		dashboard_set_property $dash set_dest_add enabled true
		
		dashboard_set_property $dash source_Add editable 1 
		dashboard_set_property $dash dest_Add editable 1 
		dashboard_set_property $dash pkt_byte_size_val editable 1 
		dashboard_set_property $dash pkt_num_val editable 1 
		dashboard_set_property $dash pkt_byte_size_begin_val editable 1 
		dashboard_set_property $dash pkt_byte_size_end_val editable 1 
		dashboard_set_property $dash pkt_incr_num_val editable 1 
			
		
	}
	
}



proc proc_set_dest_add {} {

                global dash
                
                set da [dashboard_get_property $dash dest_Add text]
                reg_write 0x1011 [expr $da & 0xffffffff]
                reg_write 0x1012 [expr $da >> 32]
				
				read_da_add
}

proc read_da_add {} {

                global dash
                
                set da1 [reg_read 0x1011]
                set da2 [reg_read 0x1012]
                set da "0x[concat "[format %0.4x [expr {$da2 & 0xffff}]][format %0.8x [expr $da1]]"]"
				
				dashboard_set_property $dash dest_Add text $da

}

proc proc_set_source_add {} {

                global dash
                
                set sa [dashboard_get_property $dash source_Add text]
                reg_write 0x1013 [expr $sa & 0xffffffff]
                reg_write 0x1014 [expr $sa >> 32]
				
				read_sa_add
}

proc read_sa_add {} {

                global dash
                
                set sa1 [reg_read 0x1013]
                set sa2 [reg_read 0x1014]
                set sa "0x[concat "[format %0.4x [expr {$sa2 & 0xffff}]][format %0.8x [expr $sa1]]"]"
				
				dashboard_set_property $dash source_Add text $sa

}

proc iadpt_control { base } {

	global iadpt_set
	
	
	if {$iadpt_set} { 
	#puts "Trigger Adapt"
	reg_rmw_adp $base 0x148 0x1 0x0
	after 100
	reg_rmw_adp $base 0x148 0x1 0x1
	reg_rmw_adp $base 0x14c 0x1 0x1
	} else {
	reg_rmw_adp $base 0x148 0x1 0x0
	after 100	
	reg_rmw_adp $base 0x14c 0x1 0x0
	}
}



